import os
import subprocess

import docker
import typer

from rubbrband.clients.docker_client import pull_image_handler
from rubbrband.controllers.models.control.train import main as train_control
from rubbrband.controllers.models.dreambooth.train import main as train_dreambooth
from rubbrband.controllers.models.lora.train import main as train_lora

db = {}
client = None
app = typer.Typer(no_args_is_help=True)


@app.callback()
def main():
    """
    Train a MODEL :robot:

    Example: rubbrband train lora
    """


@app.command(rich_help_panel="Models :robot:", help="Stable diffusion models, trained with dreambooth method")
def dreambooth(
    ctx: typer.Context,
    class_word: str = typer.Option(
        ..., help="The name that you want to give to the class of images that you'll want to generate"
    ),
    reg_dir: str = typer.Option(..., help="The full path that contains the regularization images."),
    dataset_dir: str = typer.Option(..., help="The full path that contains the images you want to finetune on"),
    model_name: str = "The name you want to give your model checkpoint file",
):
    train(ctx, "dreambooth")


@app.command(rich_help_panel="Models :robot:", help="Low-rank adapation for efficient stable diffusion fine tuning")
def lora(
    ctx: typer.Context,
    dataset_dir: str = typer.Option(..., help="The full path that contains the images you want to finetune on"),
):
    train(ctx, "lora")


@app.command(rich_help_panel="Models :robot:", help="Control diffusion models by adding extra conditions")
def control(
    ctx: typer.Context,
    dataset_dir: str = typer.Option(..., help="The full path that contains the images you want to finetune on"),
):
    train(ctx, "control")


def handle_model_train(params: dict, model: str):
    """Handle the training of a model."""

    if model == "control":
        train_control(**params)
    elif model == "dreambooth":
        train_dreambooth(**params)
    elif model == "lora":
        train_lora()
    else:
        typer.echo(f"Model {model} is not supported.")


def train(ctx: typer.Context, model: str):
    """Entrypoint for training a model."""
    if model not in db:
        typer.echo("Model not found")
        return

    image_name = f"rubbrband/{model}"
    container_name = f"rb-{model}"
    pull_image_handler(image_name)

    abs_path = os.path.abspath(ctx.params["dataset_dir"])

    try:
        container = client.containers.get(container_name)

        # stop and remove container if it is already running
        if container.status == "running":
            container.stop()
            container.remove()

    except docker.errors.NotFound:
        pass
    try:
        subprocess.check_output("nvidia-smi")
        device_requests = [docker.types.DeviceRequest(count=-1, capabilities=[["gpu"]])]
    except (
        Exception
    ):  # this command not being found can raise quite a few different errors depending on the configuration
        device_requests = []

    client.containers.run(
        image_name,
        device_requests=device_requests,
        detach=True,
        name=container_name,
        volumes={f"{abs_path}": {"bind": "/home/engineering/data", "mode": "rw"}},
        tty=True,
        stdin_open=True,
    )
    container = client.containers.get(f"rb-{model}")

    if container.status != "running":
        container.start()

    handle_model_train(ctx.params, model)


if __name__ == "__main__":
    app()



import subprocess

import docker
import typer

from rubbrband.controllers.models.control.eval import main as eval_control
from rubbrband.controllers.models.dreambooth.eval import main as eval_dreambooth
from rubbrband.controllers.models.lora.eval import main as eval_lora

db = {}
client = None
app = typer.Typer(no_args_is_help=True)


@app.callback()
def main():
    """
    Evaluate a trained MODEL :robot:

    Example: rubbrband eval lora
    """
    pass


@app.command(rich_help_panel="Models :robot:", help="Stable diffusion models, trained with dreambooth method")
def dreambooth(
    ctx: typer.Context,
    input_prompt: str = typer.Option(..., help="The prompt to input into the trained model."),
    logdir: str = "experiment_logs",
):
    eval(ctx, "dreambooth")


@app.command(rich_help_panel="Models :robot:", help="Low-rank adapation for efficient stable diffusion fine tuning")
def lora(
    ctx: typer.Context,
    input_prompt: str = typer.Option(..., help="The prompt to input into the trained model."),
):
    eval(ctx, "lora")


def control_callback(annotator_type: str):
    """Callback for the control command."""
    valid_annotators = ["canny", "depth", "hed", "mlsd", "normal", "openpose", "scribble", "seg"]

    if annotator_type not in valid_annotators:
        typer.echo(f"Invalid annotator type. Valid types are: {', '.join(valid_annotators)}")
        return


@app.command(rich_help_panel="Models :robot:", help="Low-rank adapation for efficient stable diffusion fine tuning")
def control(
    ctx: typer.Context,
    annotator_type: str = typer.Option(..., callback=control_callback, help="The edge detector to use."),
):
    eval(ctx, "control")


def handle_model_eval(params: dict, model: str):
    """Handle which model to run."""

    if model == "control":
        eval_control(**params)
    elif model == "dreambooth":
        eval_dreambooth(**params)
    elif model == "lora":
        eval_lora(**params)
    else:
        typer.echo(f"Model {model} is not supported.")

    # move file from docker container to local directory
    # the file is located at /home/engineering/samples/output.jpg
    # the file is moved to the current directory
    subprocess.run(["docker", "cp", f"rb-{model}:/home/engineering/samples", "."])


def eval(ctx: typer.Context, model: str):
    """Entrypoint for evaluating a model."""

    try:
        container = client.containers.get(f"rb-{model}")
    except docker.errors.NotFound:
        typer.echo(f"Container rb-{model} does not exist. Please train the model first.")
        return

    if container.status != "running":
        container.start()

    handle_model_eval(ctx.params, model)
    typer.echo("Inference complete. Check the current directory for the output image.")


if __name__ == "__main__":
    app()
